use io;

/* Read instance data */
function input() {
    usage = "Usage: localsolver smallest_circle.lsp "
            + "inFileName=inputFile [solFileName=outputFile] [lsTimeLimit=timeLimit]";

    if (inFileName == nil) throw usage;
    local inFile = io.openRead(inFileName);

    nbPoints = inFile.readInt();
    for [i in 0...nbPoints] {
        coordX[i] = inFile.readInt();
        coordY[i] = inFile.readInt();
    }

    minX = min[i in 0...nbPoints](coordX[i]);
    minY = min[i in 0...nbPoints](coordY[i]);
    maxX = max[i in 0...nbPoints](coordX[i]);
    maxY = max[i in 0...nbPoints](coordY[i]);
}

/* Declare the optimization model */
function model() {

    // x, y are respectively the abscissa and the ordinate of the origin of the circle
    x <- float(minX, maxX);
    y <- float(minY, maxY);

    // Minimize the radius
    r <- sqrt(max[i in 0...nbPoints](pow(x - coordX[i], 2) + pow(y - coordY[i], 2)));
    minimize r;
}

/* Parametrize the solver */
function param() {
    if (lsTimeLimit == nil) lsTimeLimit = 6;
}

/* Write the solution in a file */
function output() {
    if (solFileName != nil) {
        println("Write solution into file '" + solFileName + "'");
        local solFile = io.openWrite(solFileName);
        solFile.println("x=", x.value);
        solFile.println("y=", y.value);
        solFile.println("r=", r.value);
    }
}
